VERSION 5.00
Begin VB.Form Form1 
   Caption         =   "Form1"
   ClientHeight    =   780
   ClientLeft      =   120
   ClientTop       =   450
   ClientWidth     =   2400
   LinkTopic       =   "Form1"
   ScaleHeight     =   780
   ScaleWidth      =   2400
   StartUpPosition =   3  'Windows Default
   Begin VB.CommandButton Command1 
      Caption         =   "Command1"
      Height          =   615
      Left            =   120
      TabIndex        =   0
      Top             =   90
      Width           =   2175
   End
End
Attribute VB_Name = "Form1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 11/2018

Option Explicit

' Info ber weitere Gif-Metadaten die geschrieben/gelesen werden knnen.
' https://docs.microsoft.com/de-de/windows/desktop/wic/-wic-native-image-format-metadata-queries#gif-metadata

' einige Metadaten, die ber den IWICMetadataQueryWriter geschrieben werden knnen, werden beim
' schreiben eines Frames in die GIF von WIC wieder berschrieben. Workaround bisher nachtrglich
' einige Werte ber den IStream zu schreiben.

' Das fertige GIF enthlt nur lokale Farbtabellen weswegen es keinen Sinn macht entsprechende Metadaten
' fr eine globale Farbtabelle zu schreiben.

Private pIFactory As Long
Private sBitmaps(9) As String

Private Sub Form_Load()

    Dim lItem As Long
    Dim lCount As Long

    ' WICImagingFactory-Interface erstellen (Version 1)
    If WICCreateImagingFactory_Proxy(WINCODEC_SDK_VERSION_1, pIFactory) <> S_OK Then

        Debug.Print "WICImagingFactory konnte nicht erstellt werden!"

    Else

        ' Bilder, die in die animierte GIF sollen.
        ' alle Bilder haben die gleiche Gre!!!
        lCount = UBound(sBitmaps)
        
        For lItem = 0 To lCount

            sBitmaps(lItem) = App.Path & "\Images\Frame_" & String$(3 - Len(CStr(lItem)), _
                "0") & CStr(lItem) & ".png"

        Next

    End If

End Sub

Private Sub Form_Unload(Cancel As Integer)

    ' ist ein WICImagingFactory-Interface vorhanden
    If pIFactory <> 0 Then

        ' WICImagingFactory-Interface freigeben
        Call ReleaseInterface(pIFactory)

    End If

End Sub

Private Sub Command1_Click()

    Dim sAnimGif As String
    Dim lItem As Long
    Dim lFrames As Long
    Dim pIStream As Long
    Dim pIWICStream As Long
    Dim pIEncoder As Long
    Dim pIBitmapSource As Long

    ' Gif-Datei, die erstellt werden soll
    sAnimGif = App.Path & "\test.gif"

    ' ist ein WICImagingFactory-Interface vorhanden
    If pIFactory <> 0 Then

        ' Gif-Datei erstellen und davon einen Gif-Encoder
        pIEncoder = CreateGifEncoder(sAnimGif, pIFactory, pIWICStream, pIStream)

        ' ab hier knnte man sich in den WIC-Stream oder File-Stream einklinken
        ' und weitere Werte schreiben/manipulieren.

        ' ist ein Encoder vorhanden
        If pIEncoder <> 0 Then

            ' optional
            ' Signatur = GIF87a oder GIF89a
            ' Standard wird GIF89a geschrieben, auch ohne diese Sub.
            Call WriteSignature(pIEncoder, "GIF89a")

            ' erforderlich!!!
            ' Metadaten fr den Application Extension Block und LoopCount schreiben
            ' Loop = 0 (endlos Loop), max = 65535 (&HFFFF&)
            Call WriteAppExtAndLoopCount(pIEncoder, 0&)

            ' Anzahl der Bitmaps im Array
            lFrames = UBound(sBitmaps)

            ' alle Bitmaps durchlaufen
            For lItem = 0 To lFrames

                ' Image laden -> pIBitmapSource
                pIBitmapSource = LoadImage(pIFactory, sBitmaps(lItem))

                ' ist ein Image vorhanden
                If pIBitmapSource <> 0 Then

                    Debug.Print "Schreibe Frame " & CStr(lItem + 1) & " von " & _
                        CStr(lFrames + 1)

                    ' Image in das Gif schreiben.
                    ' optional FrameDelay, 0 = Standard 100ms, min = 1 (10ms),
                    ' max = 65535 (655350ms)
                    ' optional FrameComment
                    Call WriteGifFrame(pIBitmapSource, pIEncoder, 10, "Das ist " & _
                        "der Kommentar zum Frame " & CStr(lItem + 1))

                    ' Image freigeben
                    Call ReleaseInterface(pIBitmapSource)

                End If
                
            Next

            ' nderungen am Encoder speichern
            Call IWICBitmapEncoder_Commit_Proxy(pIEncoder)

            ' Encoder freigeben
            Call ReleaseInterface(pIEncoder)

            ' WIC-Stream freigeben
            Call ReleaseInterface(pIWICStream)



            ' Manipulieren und schreiben von Werten in den FileStream ;-)
            
            ' mindestens die Gre des grten Frames oder grer. max = &HFFFF&
            ' kleinere Frames knnen dann entsprechend positioniert werden.
            ' WIC schreibt von sich aus den LogicalScreenSize. Ein schreiben ber
            ' des LogicalScreenSize ber die Metadaten geht zwar, wird aber beim schreiben
            ' eines Frames wieder berschrieben. Kann aber an dieser Stelle manipuliert werden.
            Call WriteLogicalScreenSize(pIStream, 200, 200)
            
            ' schreibt einen Kommentar an das Ende der animierten GIF
            ' ist so von WIC nicht vorgesehen. ;-)
            Call WriteTrailingComment(pIStream, "Die animierte GIF wurde mit " & _
                "der Windows Imaging Component (WIC) erstellt. Autor: F. " & _
                "Schler (frank@activevb.de) fr ActiveVB.")

            ' File-Stream freigeben
            Call ReleaseInterface(pIStream)

        End If

    End If

    Debug.Print "Fertig!"

End Sub

Private Sub WriteLogicalScreenSize(ByVal pIStream As Long, ByVal width As Long, _
    ByVal height As Long)

    Dim bData(3) As Byte

    If width < &H0& Then width = Abs(width)
    If width > &HFFFF& Then width = &HFFFF&
    If height < &H0& Then height = Abs(height)
    If height > &HFFFF& Then height = &HFFFF&

    Call CopyMemory(bData(0), width, 2) ' LogicalScreenWidth (VT_UI2)
    Call CopyMemory(bData(2), height, 2) ' LogicalScreenHeight (VT_UI2)

    ' kommt gleich nach "GIF89a" an Position 6
    Call IStream_Seek(pIStream, 6, STREAM_SEEK_SET)

    ' bData in den Stream schreiben
    Call IStream_Write(pIStream, bData, UBound(bData) + 1)

End Sub

Private Sub WriteTrailingComment(ByVal pIStream As Long, Optional ByVal Comment As _
    String = vbNullString)

    Dim lCommentSize As Long
    Dim bData() As Byte

    If Comment <> vbNullString Then

        If Len(Comment) > 255 Then Comment = Mid$(Comment, 1, 255)

        lCommentSize = Len(Comment)

        ReDim bData(lCommentSize + 4)

        ' Comment Extension Block
        bData(0) = &H21 ' Extension Introducer
        bData(1) = &HFE ' Comment Label
        bData(2) = lCommentSize ' Block Size
        Call CopyMemory(bData(3), ByVal Comment, lCommentSize) ' Comment Data
        bData(lCommentSize + 3) = &H0& ' Block Terminator
        bData(lCommentSize + 4) = &H3B& ' Trailer

        ' an das Ende des Streams - 1 Byte (Trailer) springen
        Call IStream_Seek(pIStream, IStream_Seek(pIStream, 0, STREAM_SEEK_END) - _
            1, STREAM_SEEK_SET)

        ' bData in den Stream schreiben
        Call IStream_Write(pIStream, bData, UBound(bData) + 1)

    End If

End Sub

Private Function CreateGifEncoder(ByVal sGifFile As String, ByVal pIImagingFactory _
    As Long, ByRef pIWICStream As Long, ByRef pIStream As Long) As Long

    Dim pRet As Long
    Dim pIEncoder As Long
    Dim psGifFile As Long

    psGifFile = StrPtr(sGifFile)

    ' einen File-Stream erstellen -> pIStream
    If SHCreateStreamOnFileEx(psGifFile, STGM_CREATE Or STGM_WRITE, 0&, 0&, 0&, _
        pIStream) = S_OK Then

        ' einen WIC-Stream erstellen -> pIWICStream
        If IWICImagingFactory_CreateStream_Proxy(pIImagingFactory, pIWICStream) = _
            S_OK Then

            ' WIC-Stream mit dem File-Stream initialisieren
            If IWICStream_InitializeFromIStream_Proxy(pIWICStream, pIStream) = _
                S_OK Then

                ' einen GIF-Encoder erstellen -> pIEncoder
                If IWICImagingFactory_CreateEncoder_Proxy(pIImagingFactory, _
                    String2Guid(GUID_ContainerFormatGif), 0&, pIEncoder) = S_OK _
                    Then

                    ' Encoder vom WIC-Stream initialisieren
                    If IWICBitmapEncoder_Initialize_Proxy(pIEncoder, pIWICStream, _
                        WICBitmapEncoderNoCache) = S_OK Then

                        pRet = pIEncoder

                    End If
                End If
            End If
        End If
    End If

    CreateGifEncoder = pRet

End Function

Private Function LoadImage(ByVal pIImagingFactory As Long, ByVal FileName As _
    String) As Long

    Dim lFrameCount As Long
    Dim pFileName As Long
    Dim pIDecoder As Long
    Dim pIBitmapSource As Long
    Dim pIBitmapDecodeFrame As Long

    ' existiert die Datei
    If FileExists(FileName) = True Then

        pFileName = StrPtr(FileName)

        ' einen Decoder von der Bilddatei erstellen -> pIDecoder
        If IWICImagingFactory_CreateDecoderFromFilename_Proxy(pIImagingFactory, _
            pFileName, 0&, GENERIC_READ, WICDecodeMetadataCacheOnLoad, pIDecoder) _
            = S_OK Then

            ' Anzahl der Frames in dem Bild ermitteln
            If IWICBitmapDecoder_GetFrameCount_Proxy(pIDecoder, lFrameCount) = _
                S_OK Then

                ' mindestens 1 Frame muss vorhanden sein
                If lFrameCount > 0 Then

                    ' erstes Frame aus dem Bild laden -> pIBitmapDecodeFrame
                    If IWICBitmapDecoder_GetFrame_Proxy(pIDecoder, 0&, _
                        pIBitmapDecodeFrame) = S_OK Then

                        ' Frame in das 8bppIndexed Format konvertieren
                        pIBitmapSource = ImageTo8bppIndexed(pIImagingFactory, _
                            pIBitmapDecodeFrame, WICBitmapDitherTypeNone, _
                            WICBitmapPaletteTypeMedianCut)

                        ' Frame freigeben
                        Call ReleaseInterface(pIBitmapDecodeFrame)

                    End If
                End If
            End If

            ' Decoder freigeben
            Call ReleaseInterface(pIDecoder)

        End If

    End If

    LoadImage = pIBitmapSource

End Function

Private Function ImageTo8bppIndexed(ByVal pIImagingFactory As Long, ByVal _
    pIWICBitmapSource As Long, Optional ByVal DitherType As WICBitmapDitherType = _
    WICBitmapDitherTypeNone, Optional ByVal PaletteType As WICBitmapPaletteType = _
    WICBitmapPaletteTypeMedianCut) As Long

    Dim pPalette As Long
    Dim pIBitmapSource As Long
    Dim pFormatConverter As Long

    ' einen FormatConverter erstellen
    If IWICImagingFactory_CreateFormatConverter_Proxy(pIImagingFactory, _
        pFormatConverter) = S_OK Then

        ' eine Palette erstellen
        If IWICImagingFactory_CreatePalette_Proxy(pIImagingFactory, pPalette) = _
            S_OK Then

            ' Palette von Bitmap mit 256 Farben initialisieren
            If IWICPalette_InitializeFromBitmap_Proxy(pPalette, pIWICBitmapSource, _
                256, True) = S_OK Then

                ' Bitmap in das 8bppIndexed-Format mit entsprechendem
                ' DitherType und PaletteType konvertieren
                If IWICFormatConverter_Initialize_Proxy(pFormatConverter, _
                    pIWICBitmapSource, String2Guid( _
                    GUID_WICPixelFormat8bppIndexed), DitherType, pPalette, 0, _
                    PaletteType) = S_OK Then

                    ' konvertiertes Bitmap vom FormatConverter holen
                    pIBitmapSource = QueryInterface(pFormatConverter, _
                        IID_IWICBitmapSource)

                End If

            End If

            ' Palette freigeben
            Call ReleaseInterface(pPalette)

        End If

        ' FormatConverter freigeben
        Call ReleaseInterface(pFormatConverter)

    End If

    ImageTo8bppIndexed = pIBitmapSource

End Function

Private Sub WriteGifFrame(ByVal pIWICBitmapSource As Long, ByVal pIWICBitmapEncoder As _
    Long, Optional ByVal FrameDelay As Long = 0, Optional ByVal FrameComment As _
    String = vbNullString)

    Dim lWidth As Long
    Dim lHeight As Long
    Dim tWICRect As WICRect
    Dim pIBitmapEncodeFrame As Long
    
    ' dem Encoder ein neues Frame hinzufgen -> pIBitmapEncodeFrame
    If IWICBitmapEncoder_CreateNewFrame_Proxy(pIWICBitmapEncoder, pIBitmapEncodeFrame, 0&) _
        = S_OK Then

        ' neues Frame initialisieren
        If IWICBitmapFrameEncode_Initialize_Proxy(pIBitmapEncodeFrame, 0&) = S_OK _
            Then

            ' Gre der BitmapSource ermitteln
            If IWICBitmapSource_GetSize_Proxy(pIWICBitmapSource, lWidth, lHeight) _
                = S_OK Then

                ' entsprechend Gre fr das neue GIF-Frame setzen
                If IWICBitmapFrameEncode_SetSize_Proxy(pIBitmapEncodeFrame, _
                    lWidth, lHeight) = S_OK Then

                    ' erforderlich!!!
                    ' Schreibt den Delay fr den Frame
                    Call WriteFrameDelay(pIBitmapEncodeFrame, FrameDelay)

                    ' optional
                    ' Schreibt einen Kommentar in den Frame
                    Call WriteFrameComment(pIBitmapEncodeFrame, FrameComment)
                    
                    ' optional
                    ' legt fest das es eine Transparente Farbe gibt
                    ' und welche Farbe aus der Palette das sein soll.
                    Call WriteTransparency(pIBitmapEncodeFrame, True, 0)
                    
                    ' optional
                    ' legt die Position (Left, Top) beim rendern fest. Macht nur Sinn wenn
                    ' das erste Frame grer ist als die restlichen Frames oder wenn
                    ' Logical Screen Width/Height grer ist als die grte Bitmap!
                    ' Max Left = Logical Screen Width - Frame Width
                    ' Max Top = Logical Screen Height - Frame Heigh
                    Call WriteFramePos(pIBitmapEncodeFrame, 0, 0)
                    
                    ' optional
                    ' legt fest wie die einzelnen Frames spter gerendert werden sollen.
                    Call WriteDisposalMethod(pIBitmapEncodeFrame, DM_UNDEFINED)
                    
                    With tWICRect
                        .x = 0
                        .y = 0
                        .width = lWidth
                        .height = lHeight
                    End With

                    ' BitmapSource in das GIF-Frame schreiben
                    If IWICBitmapFrameEncode_WriteSource_Proxy( _
                        pIBitmapEncodeFrame, pIWICBitmapSource, tWICRect) = S_OK _
                        Then
                        
                        ' erst mit WriteSource werden werden Metadaten und das Bild in
                        ' die GIF geschrieben. Commit schliet dann den Frame ab so das
                        ' das nchste Bild als neuer Frame geschrieben wird.
                        
                        ' nderungen am Frame speichern
                        Call IWICBitmapFrameEncode_Commit_Proxy(pIBitmapEncodeFrame)

                    End If

                End If

            End If

        End If

        ' GIF-Frame freigeben
        Call ReleaseInterface(pIBitmapEncodeFrame)

    End If

End Sub

Private Sub WriteAppExtAndLoopCount(ByVal pIWICBitmapEncoder As Long, Optional ByVal LoopCount _
    As Long = 0)
    
    Dim bMetaData() As Byte
    Dim sMetadataName As String
    Dim psMetadataName As Long
    Dim ptPROPVARIANT_ARRAY As Long
    Dim pIMetadataQueryWriter As Long
    Dim tPROPVARIANT_ARRAY As PROPVARIANT_ARRAY

    ' MetadataWriter vom Encoder erstellen
    If IWICBitmapEncoder_GetMetadataQueryWriter_Proxy(pIWICBitmapEncoder, _
        pIMetadataQueryWriter) = S_OK Then

        ' Metadaten fr den Application Extension Block
        ReDim bMetaData(10)
        Call CopyMemory(bMetaData(0), ByVal "NETSCAPE2.0", 11)

        sMetadataName = "/appext/Application"
        tPROPVARIANT_ARRAY.vt = VARENUM.VT_UI1 Or VARENUM.VT_VECTOR
        tPROPVARIANT_ARRAY.vData.cElems = 11
        tPROPVARIANT_ARRAY.vData.pElems = VarPtr(bMetaData(0))
        
        psMetadataName = StrPtr(sMetadataName)
        ptPROPVARIANT_ARRAY = VarPtr(tPROPVARIANT_ARRAY)

        ' Metadaten schreiben
        If IWICMetadataQueryWriter_SetMetadataByName_Proxy(pIMetadataQueryWriter, _
            psMetadataName, ptPROPVARIANT_ARRAY) = S_OK Then

            'Debug.Print "Write /appext/application"

        End If

        ' Metadaten fr den Application Extension Block (Loop)
        If LoopCount < &H0& Then LoopCount = Abs(LoopCount)
        If LoopCount > &HFFFF& Then LoopCount = &HFFFF&
        
        ReDim bMetaData(4)
        bMetaData(0) = 3 ' ExtSize (muss grer als 1 sein)
        bMetaData(1) = 1 ' LoopType (1 = Animierte GIF)
        
        ' LoopCount in das ByteArray kopieren
        Call CopyMemory(bMetaData(2), LoopCount, 2)
        
        bMetaData(4) = 0 ' Ende des Datenblocks
        
        sMetadataName = "/appext/Data"
        tPROPVARIANT_ARRAY.vt = VARENUM.VT_UI1 Or VARENUM.VT_VECTOR
        tPROPVARIANT_ARRAY.vData.cElems = 5
        tPROPVARIANT_ARRAY.vData.pElems = VarPtr(bMetaData(0))

        psMetadataName = StrPtr(sMetadataName)
        ptPROPVARIANT_ARRAY = VarPtr(tPROPVARIANT_ARRAY)

        ' Metadaten schreiben
        If IWICMetadataQueryWriter_SetMetadataByName_Proxy(pIMetadataQueryWriter, _
            psMetadataName, ptPROPVARIANT_ARRAY) = S_OK Then

            'Debug.Print "Write /appext/data"

        End If
        
        ' MetadataWriter freigeben
        Call ReleaseInterface(pIMetadataQueryWriter)

    End If

End Sub

Private Sub WriteSignature(ByVal pIWICBitmapEncoder As Long, Optional ByVal _
    Signature As String = "GIF89a")

    Dim bMetaData() As Byte
    Dim sMetadataName As String
    Dim psMetadataName As Long
    Dim ptPROPVARIANT As Long
    Dim ptPROPVARIANT_ARRAY As Long
    Dim pIMetadataQueryWriter As Long
    Dim tPROPVARIANT As PROPVARIANT
    Dim tPROPVARIANT_ARRAY As PROPVARIANT_ARRAY

    ' nur wenn eine Signatur vorhanden ist
    If Signature <> vbNullString Then

        ' nur wenn es sich um eine gltige Signatur handelt
        If Signature = "GIF87a" Or Signature = "GIF89a" Then

            ' MetadataWriter vom Encoder erstellen
            If IWICBitmapEncoder_GetMetadataQueryWriter_Proxy(pIWICBitmapEncoder, _
                pIMetadataQueryWriter) = S_OK Then

                ' Metadaten fr die Signatur
                ReDim bMetaData(5)

                Call CopyMemory(bMetaData(0), ByVal Signature, 6)

                sMetadataName = "/logscrdesc/Signature"
                tPROPVARIANT_ARRAY.vt = VARENUM.VT_UI1 Or VARENUM.VT_VECTOR
                tPROPVARIANT_ARRAY.vData.cElems = 6
                tPROPVARIANT_ARRAY.vData.pElems = VarPtr(bMetaData(0))

                psMetadataName = StrPtr(sMetadataName)
                ptPROPVARIANT_ARRAY = VarPtr(tPROPVARIANT_ARRAY)

                ' Metadaten schreiben
                If IWICMetadataQueryWriter_SetMetadataByName_Proxy( _
                    pIMetadataQueryWriter, psMetadataName, ptPROPVARIANT_ARRAY) = _
                    S_OK Then

                    ' Debug.Print "Write /logscrdesc/Signature"

                End If

                ' MetadataWriter freigeben
                Call ReleaseInterface(pIMetadataQueryWriter)

            End If

        End If

    End If

End Sub

Private Sub WriteFrameComment(ByVal pIWICBitmapFrameEncode As Long, Optional ByVal _
    Comment As String = vbNullString)

    Dim sMetadataName As String
    Dim psMetadataName As Long
    Dim ptPROPVARIANT As Long
    Dim pIMetadataQueryWriter As Long
    Dim tPROPVARIANT As PROPVARIANT

    ' nur wenn ein Kommentar vorhanden ist
    If Comment <> vbNullString Then

        ' MetadataWriter vom Frame erstellen
        If IWICBitmapFrameEncode_GetMetadataQueryWriter_Proxy( _
            pIWICBitmapFrameEncode, pIMetadataQueryWriter) = S_OK Then
            
            ' max 255 Zeichen
            If Len(Comment) > 255 Then Comment = Mid$(Comment, 1, 255)
            
            ' Metadaten fr einen Kommentar
            sMetadataName = "/commentext/TextEntry"
            tPROPVARIANT.vt = VARENUM.VT_LPSTR
            tPROPVARIANT.vData = String2LPSTR(Comment)

            psMetadataName = StrPtr(sMetadataName)
            ptPROPVARIANT = VarPtr(tPROPVARIANT)

            ' Metadaten schreiben
            If IWICMetadataQueryWriter_SetMetadataByName_Proxy( _
                pIMetadataQueryWriter, psMetadataName, ptPROPVARIANT) = S_OK Then

                ' Debug.Print "Write /commentext/TextEntry"

            End If

            ' MetadataWriter freigeben
            Call ReleaseInterface(pIMetadataQueryWriter)

        End If

    End If

End Sub

Private Sub WriteFrameDelay(ByVal pIWICBitmapFrameEncode As Long, Optional ByVal _
    Delay As Long = 0)

    Dim sMetadataName As String
    Dim psMetadataName As Long
    Dim ptPROPVARIANT As Long
    Dim pIMetadataQueryWriter As Long
    Dim tPROPVARIANT As PROPVARIANT

    ' MetadataWriter vom Frame erstellen
    If IWICBitmapFrameEncode_GetMetadataQueryWriter_Proxy(pIWICBitmapFrameEncode, _
        pIMetadataQueryWriter) = S_OK Then
        
        If Delay = &H0& Then Delay = 10
        If Delay < &H0& Then Delay = Abs(Delay)
        If Delay > &HFFFF& Then Delay = &HFFFF&
        
        ' Metadaten fr Delay
        sMetadataName = "/grctlext/Delay"
        tPROPVARIANT.vt = VARENUM.VT_UI2
        tPROPVARIANT.vData = Delay

        psMetadataName = StrPtr(sMetadataName)
        ptPROPVARIANT = VarPtr(tPROPVARIANT)

        ' Metadaten schreiben
        If IWICMetadataQueryWriter_SetMetadataByName_Proxy(pIMetadataQueryWriter, _
            psMetadataName, ptPROPVARIANT) = S_OK Then
            
            'Debug.Print "Write /grctlext/Delay"

        End If

        ' MetadataWriter freigeben
        Call ReleaseInterface(pIMetadataQueryWriter)

    End If

End Sub

Private Sub WriteTransparency(ByVal pIWICBitmapFrameEncode As Long, Optional ByVal _
    Transparency As Boolean = False, Optional ByVal PaletteIndex As Byte = 0)

    Dim sMetadataName As String
    Dim psMetadataName As Long
    Dim ptPROPVARIANT As Long
    Dim pIMetadataQueryWriter As Long
    Dim tPROPVARIANT As PROPVARIANT

    ' nur wenn das Frame eine Transparente Farbe enthalten soll
    ' Standard ist bei /grctlext/TransparencyFlag = False
    If Transparency = True Then

        ' MetadataWriter vom Frame erstellen
        If IWICBitmapFrameEncode_GetMetadataQueryWriter_Proxy( _
            pIWICBitmapFrameEncode, pIMetadataQueryWriter) = S_OK Then

            ' Metadaten fr TransparencyFlag
            sMetadataName = "/grctlext/TransparencyFlag"
            tPROPVARIANT.vt = VARENUM.VT_BOOL
            tPROPVARIANT.vData = True

            psMetadataName = StrPtr(sMetadataName)
            ptPROPVARIANT = VarPtr(tPROPVARIANT)

            ' Metadaten schreiben
            If IWICMetadataQueryWriter_SetMetadataByName_Proxy( _
                pIMetadataQueryWriter, psMetadataName, ptPROPVARIANT) = S_OK Then

                ' Debug.Print "Write /grctlext/TransparencyFlag"

            End If

            ' Metadaten fr TransparentColorIndex
            sMetadataName = "/grctlext/TransparentColorIndex"
            tPROPVARIANT.vt = VARENUM.VT_UI1
            tPROPVARIANT.vData = PaletteIndex

            psMetadataName = StrPtr(sMetadataName)
            ptPROPVARIANT = VarPtr(tPROPVARIANT)

            ' Metadaten schreiben
            If IWICMetadataQueryWriter_SetMetadataByName_Proxy( _
                pIMetadataQueryWriter, psMetadataName, ptPROPVARIANT) = S_OK Then

                ' Debug.Print "Write /grctlext/TransparentColorIndex"

            End If

            ' MetadataWriter freigeben
            Call ReleaseInterface(pIMetadataQueryWriter)

        End If

    End If

End Sub

Private Sub WriteDisposalMethod(ByVal pIWICBitmapFrameEncode As Long, Optional _
    ByVal Disposal As DISPOSAL_METHODS = DM_UNDEFINED)

    Dim sMetadataName As String
    Dim psMetadataName As Long
    Dim ptPROPVARIANT As Long
    Dim pIMetadataQueryWriter As Long
    Dim tPROPVARIANT As PROPVARIANT

    ' nur wenn Disposal <> DM_UNDEFINED ist
    ' Standard ist bei /grctlext/Disposal = DM_UNDEFINED
    If Disposal <> DM_UNDEFINED Then

        ' MetadataWriter vom Frame erstellen
        If IWICBitmapFrameEncode_GetMetadataQueryWriter_Proxy( _
            pIWICBitmapFrameEncode, pIMetadataQueryWriter) = S_OK Then

            ' Metadaten fr Disposal
            sMetadataName = "/grctlext/Disposal"
            tPROPVARIANT.vt = VARENUM.VT_UI1
            tPROPVARIANT.vData = Disposal

            psMetadataName = StrPtr(sMetadataName)
            ptPROPVARIANT = VarPtr(tPROPVARIANT)

            ' Metadaten schreiben
            If IWICMetadataQueryWriter_SetMetadataByName_Proxy( _
                pIMetadataQueryWriter, psMetadataName, ptPROPVARIANT) = S_OK Then

                ' Debug.Print "Write /grctlext/Disposal"

            End If

            ' MetadataWriter freigeben
            Call ReleaseInterface(pIMetadataQueryWriter)

        End If
        
    End If

End Sub

Private Sub WriteFramePos(ByVal pIWICBitmapFrameEncode As Long, Optional ByVal _
    LeftPos As Long = 0, Optional ByVal TopPos As Long = 0)

    Dim bMetaData() As Byte
    Dim sMetadataName As String
    Dim psMetadataName As Long
    Dim ptPROPVARIANT As Long
    Dim pIMetadataQueryWriter As Long
    Dim tPROPVARIANT As PROPVARIANT

    ' MetadataWriter vom Frame erstellen
    If IWICBitmapFrameEncode_GetMetadataQueryWriter_Proxy(pIWICBitmapFrameEncode, _
        pIMetadataQueryWriter) = S_OK Then

        If LeftPos < &H0& Then LeftPos = Abs(LeftPos)
        If LeftPos > &HFFFF& Then LeftPos = &HFFFF&

        ' Metadaten fr die linke Position
        sMetadataName = "/imgdesc/Left"
        tPROPVARIANT.vt = VARENUM.VT_UI2
        tPROPVARIANT.vData = LeftPos

        psMetadataName = StrPtr(sMetadataName)
        ptPROPVARIANT = VarPtr(tPROPVARIANT)

        ' Metadaten schreiben
        If IWICMetadataQueryWriter_SetMetadataByName_Proxy(pIMetadataQueryWriter, _
            psMetadataName, ptPROPVARIANT) = S_OK Then

            ' Debug.Print "Write /imgdesc/Left"

        End If

        ' Metadaten fr die obere Position
        If TopPos < &H0& Then TopPos = Abs(TopPos)
        If TopPos > &HFFFF& Then TopPos = &HFFFF&

        ' Metadaten fr den Application Extension Block
        sMetadataName = "/imgdesc/Top"
        tPROPVARIANT.vt = VARENUM.VT_UI2
        tPROPVARIANT.vData = TopPos

        psMetadataName = StrPtr(sMetadataName)
        ptPROPVARIANT = VarPtr(tPROPVARIANT)

        ' Metadaten schreiben
        If IWICMetadataQueryWriter_SetMetadataByName_Proxy(pIMetadataQueryWriter, _
            psMetadataName, ptPROPVARIANT) = S_OK Then

            ' Debug.Print "Write /imgdesc/Top"

        End If

        ' MetadataWriter freigeben
        Call ReleaseInterface(pIMetadataQueryWriter)

    End If

End Sub
